@@ -929,98 +929,128 @@ def time_series_tw (df):
 
     #-------------------------------------------Ratios and other advanced features----------------------------------#
 def duplicates_detection(data):
-            data["possible_duplicates"] = 0
-            data["close_duplicates"] = 0
-            data["group_duplicates"] = 0
-            close_duplicates_list =[]
-            group_duplicates_list = []
-            for i in range(len(data)):
-                if i < len(data):
-                    possible_duplicated = []
-                    for j in range(len(data)):
-                        if j !=i:
-                            list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
-                            list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
-                            distance, path = fastdtw(list_dtw_1,list_dtw_2)
-                            if (distance < 12600):
-                                possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
-                    close_duplicates= []
-                    group_duplicates = []
-                    if (len(possible_duplicated)!=0):              
-                        data["possible_duplicates"].iloc[i] = possible_duplicated                        
-                        for t in range(len(data["possible_duplicates"].iloc[i])):
-                            #text_1 is the differences possible duplicates with the profile evaluate
-                            text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
-                            text_1 = pd.DataFrame()
-                            for a in range(len(text_1_aux)):
-                                aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
-                                text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
-                            text_1 = list(itertools.chain(*text_1["text"]))   
-                            text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
-                            if "Empty" in text_1:
-                                text_1 = list(filter(("Empty").__ne__, text_1))
-                            if "Empty" in text_2:  
-                                text_2 = list(filter(("Empty").__ne__, text_2))
-                            if "" in text_1:
-                                text_1 = list(filter(("").__ne__, text_1))
-                            if "" in text_2:  
-                                text_2 = list(filter(("").__ne__, text_2))  
-                            comparacion = [item for item in text_2 if item in text_1]
-                            if len(comparacion) > 0:
-                                tw_similarity = len(comparacion)/(data["num_userTw"].iloc[i])
-                                if (tw_similarity > 0.00003): 
-                                    group_duplicates.append(data["possible_duplicates"].iloc[i][t]["screen_name"])                           
-                                    close_duplicates.append({"screen_name":data["possible_duplicates"].iloc[i][t]["screen_name"], "distance": data["possible_duplicates"].iloc[i][t]["distance"],
-                                                                "tw_similarity":tw_similarity, "tw_match_tw": comparacion})
-                        if len(close_duplicates) !=0:
-                            group_duplicates.append(data["screen_name"].iloc[i])
-                            group_duplicates_list.append(group_duplicates)
-                            close_duplicates_list.append(close_duplicates)
-                        else:
-                            group_duplicates_list.append("Empty")
-                            close_duplicates_list.append("Empty")                    
-                    else:
-                        group_duplicates_list.append("Empty")
-                        close_duplicates_list.append("Empty")
-            data["close_duplicates"] = close_duplicates_list
-            data["group_duplicates"] = group_duplicates_list
+    data = hour_days_tweets(data)
+    data["possible_duplicates"] = 0
+    data["close_duplicates"] = 0
+    data["group_duplicates"] = 0
+    close_duplicates_list =[]
+    group_duplicates_list = []
+    for i in range(len(data)):
+        if i < len(data):
+            possible_duplicated = []
+            for j in range(len(data)):
+                if j !=i:
+                    list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+                    list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+                    distance, path = fastdtw(list_dtw_1,list_dtw_2)
+                    if (distance < 12600):
+                        possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+            close_duplicates= []
+            group_duplicates = []
+            if (len(possible_duplicated)!=0):              
+                data["possible_duplicates"].iloc[i] = possible_duplicated                        
+                for t in range(len(data["possible_duplicates"].iloc[i])):
+                    #text_1 is the differences possible duplicates with the profile evaluate
+                    text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
+                    text_1 = pd.DataFrame()
+                    for a in range(len(text_1_aux)):
+                        aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
+                        text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
+                    text_1 = list(itertools.chain(*text_1["text"]))   
+                    text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
+                    if "Empty" in text_1:
+                        text_1 = list(filter(("Empty").__ne__, text_1))
+                    if "Empty" in text_2:  
+                        text_2 = list(filter(("Empty").__ne__, text_2))
+                    if "" in text_1:
+                        text_1 = list(filter(("").__ne__, text_1))
+                    if "" in text_2:  
+                        text_2 = list(filter(("").__ne__, text_2))  
+                    comparacion = [item for item in text_2 if item in text_1]
+                    if len(comparacion) > 0:
+                        tw_similarity = len(comparacion)/(data["num_userTw"].iloc[i])
+                        if (tw_similarity > 0.00003): 
+                            group_duplicates.append(data["possible_duplicates"].iloc[i][t]["screen_name"])                           
+                            close_duplicates.append({"screen_name":data["possible_duplicates"].iloc[i][t]["screen_name"], "distance": data["possible_duplicates"].iloc[i][t]["distance"],
+                                                        "tw_similarity":tw_similarity, "tw_match_tw": comparacion})
+                if len(close_duplicates) !=0:
+                    group_duplicates.append(data["screen_name"].iloc[i])
+                    group_duplicates_list.append(group_duplicates)
+                    close_duplicates_list.append(close_duplicates)
+                else:
+                    group_duplicates_list.append("Empty")
+                    close_duplicates_list.append("Empty")                    
+            else:
+                group_duplicates_list.append("Empty")
+                close_duplicates_list.append("Empty")
+    data["close_duplicates"] = close_duplicates_list
+    data["group_duplicates"] = group_duplicates_list
 
-            duplicate_profiles = data[data["group_duplicates"]!="Empty"].reset_index(drop=True)
-            group_of_duplicates_list=[]
-            for i in range(len(duplicate_profiles)):
-                group_of_duplicates_list.append(sorted(duplicate_profiles["group_duplicates"].iloc[i]))
-            group_duplicates = list(group_of_duplicates_list for group_of_duplicates_list,_ in itertools.groupby(group_of_duplicates_list))
-            data_group_duplicates = groups_duplicates_detection(group_duplicates,data)
-            return duplicate_profiles, data_group_duplicates
+    duplicate_profiles = data[data["group_duplicates"]!="Empty"].reset_index(drop=True)
+    group_of_duplicates_list=[]
+    for i in range(len(duplicate_profiles)):
+        group_of_duplicates_list.append(sorted(duplicate_profiles["group_duplicates"].iloc[i]))
+    group_duplicates = list(group_of_duplicates_list for group_of_duplicates_list,_ in itertools.groupby(group_of_duplicates_list))
+    data_group_duplicates = groups_duplicates_detection(group_duplicates,data)
+    return duplicate_profiles, data_group_duplicates
+
+def groups_duplicates_detection(group_duplicates,data):
+    data_final= {}
+    for i in range(len(group_duplicates)):
+        data_final["Group "+ str(i)]= []
+        
+        group_duplicate_tw_day_size = 0
+        tw_day_list_size=[] 
+        for j in range(len(group_duplicates[i])):                    
+            tw_day_list_size.append(len(data[data["screen_name"]==group_duplicates[i][j]]["tw_day_list"].iloc[0]))    
+            
+        group_duplicate_tw_day_size =  min(tw_day_list_size)    
+        group_info_list = []
+        for j in range(group_duplicate_tw_day_size): 
+            first={}
+            first = {"date": data[data["screen_name"]==group_duplicates[i][0]]["tw_day_list"].iloc[0][j].isoformat()}
+            for k in range(len(group_duplicates[i])):
+                label_tw_profile= group_duplicates[i][k]
+                first[label_tw_profile] = {"Número de tweets por dia": data[data["screen_name"]==group_duplicates[i][k]]["tw_per_day_list"].iloc[0][j],
+                                        "Número de tweets por hora": data[data["screen_name"]==group_duplicates[i][k]]["tweets_hour_day_list"].iloc[0][j]}
+            group_info_list.append(first)
+        data_final["Group "+str(i)].append({
+            "data": group_info_list,
+            "group_duplicates": group_duplicates[i]
+        })
+    data_groups_duplicates = {}
+    data_groups_duplicates["groups"] = data_final
+    return data_groups_duplicates
+
+def hour_days_tweets(df):
+    df["tweets_hour_day_list"] = 0
+    hour_days_tweets_list = []
+    for j in range(len(df)):
+        days = len(df.iloc[j]["tw_per_day_list"])
+        data = df.iloc[j]    
+        data["tweet_date_days_list_copy"] = copy.deepcopy(data["tweet_date_days_list"])
+        hour_days_tweets = []
+        for v in range(days):        
+            date_list_day = data['tweet_date_days_list_copy'][v]
+            if date_list_day !=["Empty"]:
+                for t in range(len(date_list_day)):
+                    date_list_day[t] = date_list_day[t].time().hour   
+                x = list(range(0,24))
+                y = []
+                for k in range(len(x)):
+                    count=0
+                    for l in range(len(date_list_day)):        
+                        if date_list_day[l] == k:
+                            count = count + 1
+                    y.append(count)
+            else:
+                y.append([0]*24)
+            hour_days_tweets.append(y)
+        hour_days_tweets_list.append(hour_days_tweets)
+        
+    df["tweets_hour_day_list"] = hour_days_tweets_list
+    return df
 
-        def groups_duplicates_detection(group_duplicates,data):
-            data_final= {}
-            for i in range(len(group_duplicates)):
-                data_final["Group "+ str(i)]= []
-                
-                group_duplicate_tw_day_size = 0
-                tw_day_list_size=[] 
-                for j in range(len(group_duplicates[i])):                    
-                    tw_day_list_size.append(len(data[data["screen_name"]==group_duplicates[i][j]]["tw_day_list"].iloc[0]))    
-                    
-                group_duplicate_tw_day_size =  min(tw_day_list_size)    
-                group_info_list = []
-                for j in range(group_duplicate_tw_day_size): 
-                    first={}
-                    first = {"date": data[data["screen_name"]==group_duplicates[i][0]]["tw_day_list"].iloc[0][j].isoformat()}
-                    for k in range(len(group_duplicates[i])):
-                        label_tw_profile= group_duplicates[i][k]+" - Número de tweets"
-                        first[label_tw_profile] = data[data["screen_name"]==group_duplicates[i][k]]["tw_per_day_list"].iloc[0][j]       
-                    group_info_list.append(first)
-                data_final["Group "+str(i)].append({
-                    "data": group_info_list,
-                    "group_duplicates": group_duplicates[i]
-                })
-    
-            data_groups_duplicates = {}
-            data_groups_duplicates["groups"] = data_final
-            return data_groups_duplicates
-	
 	
 def advanced_features(data):
     
@@ -1296,7 +1326,7 @@ def contextual_analysis_profile(data, keyword_corpus, hashtag_corpus):
       aux2 = pd.DataFrame({'text':aux["text"].iloc[i],'RT':aux["RT"].iloc[i],'lang': aux["lang"].iloc[i],"n_list_tw": aux["n_list_tw"].iloc[i]})
       tweets_df = tweets_df.append(aux2, ignore_index=False).reset_index(drop=True)
     tweets_df = tweets_df[tweets_df["RT"] !="2"].reset_index(drop=True)
-    tweets_df = sentiental_analysis_features(tweets_df)
+    tweets_df = sentimental_analysis_features(tweets_df)
     sample_data_tokenized = tokenize_tweets(tweets_df)
     tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
     tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
@@ -1932,7 +1962,7 @@ def sentimental_analysis_telegram(data):
     data["terrorist_suggestion"][(data["terrorist_suggestion"]==0)] = "Don´talk about terrorism" 
     return data
 
-def sentimental_analysis_features(data):
+def sentimental_analysis_features_telegram(data):
 	corpus =[]
 	polarities=[]
 	subjectivities=[]
